`timescale 1 ps / 1 ps

`include "svunit_defines.svh"

module divider_unit_test;
    import svunit_pkg::svunit_testcase;
    import unit_test_pkg::*;

    string name = "divider_unit_test";
    svunit_testcase svunit_ut;

    localparam real TIME_BASE = 1000000.0;
    localparam real CLOCK_400 = (TIME_BASE / 400.00);

    logic clk_400mhz_reset_n = 1'b0;
    logic clk_400mhz = 0;

    initial forever #(CLOCK_400/2) clk_400mhz = ~clk_400mhz;

    // ----------------------------------
    // ------- DUT initialization -------
    // ----------------------------------

    localparam WIDTH = 1024;

    logic start;            // start calculation
    logic busy;             // calculation in progress
    logic done;             // calculation is complete (high for one tick)
    logic valid;            // result is valid
    logic dbz;              // divide by zero
    logic [WIDTH-1:0] a;    // dividend (numerator)
    logic [WIDTH-1:0] b;    // divisor (denominator)
    logic [WIDTH-1:0] val;  // result value: quotient
    logic [WIDTH-1:0] rem;   // result: remainder
    
    divider #(
        .WIDTH  (WIDTH)
    ) dut ( // width of numbers in bits
        .clk                 (clk_400mhz),              // clock
        .rst                 (~clk_400mhz_reset_n),              // reset
        .start               (start),            // start calculation
        .busy                (busy),             // calculation in progress
        .done                (done),             // calculation is complete (high for one tick)
        .valid               (valid),            // result is valid
        .dbz                 (dbz),              // divide by zero
        .a                   (a),    // dividend (numerator)
        .b                   (b),    // divisor (denominator)
        .val                 (val),  // result value: quotient
        .rem                 (rem)   // result: remainder
    );
    
 

    function void build();
        svunit_ut = new(name);
    endfunction

    task setup();
        svunit_ut.setup();
        clk_400mhz_reset_n = 1'b0;

        #500ns;

        @(posedge clk_400mhz);
        clk_400mhz_reset_n = 1'b1;
    endtask

    task teardown();
        svunit_ut.teardown();
        clk_400mhz_reset_n = 1'b0;
    endtask

`SVUNIT_TESTS_BEGIN
    `SVTEST(simple_write_read)

    @(posedge clk_400mhz);
    a <= 'd1000000;
    b <= 'd3;

    @(posedge clk_400mhz);
    start <= 1'b1;

    @(posedge clk_400mhz);
    start <= 1'b0;

    repeat(5000) @(posedge clk_400mhz);



    `SVTEST_END
`SVUNIT_TESTS_END

endmodule