`timescale 1 ns / 1 ns

`include "svunit_defines.svh"

module key_generator_unit_test;
    import svunit_pkg::svunit_testcase;
    import unit_test_pkg::*;
    import registers_pkg::*;
    import axi4_lite_pkg::*;

    string name = "key_generator_unit_test";
    svunit_testcase svunit_ut;

    localparam real TIME_BASE = 1000.0;
    localparam real CLOCK_100 = (TIME_BASE / 100.00);

    logic reset_n    = 1'b0;
    logic clk_100mhz = 0;

    initial forever #(CLOCK_100/2) clk_100mhz = ~clk_100mhz;

    // ----------------------------------
    // ------- DUT initialization -------
    // ----------------------------------

    logic  [255:0]      p      = '0   ;
    logic  [255:0]      q      = '0   ;
    logic  [511:0]      e      = '0   ;
    logic               enable = '0   ;
    logic               result_valid  ;
    logic  [511:0]      n             ;
    logic  [511:0]      d             ;

    generator dut(
        .clk            (clk_100mhz),
        .rst            (~reset_n),
        .p              (p),
        .q              (q),
        .e              (e),
        .enable         (enable),
        .result_valid   (result_valid),
        .n              (n),
        .d              (d)
    );

    function void build();
        svunit_ut = new(name);
    endfunction

    task setup();
        svunit_ut.setup();
        reset_n = 1'b0;

        @(posedge clk_100mhz);
        reset_n = 1'b1;
    endtask

    task teardown();
        svunit_ut.teardown();
        reset_n = 1'b0;
    endtask

typedef struct packed{
    bit  [ 255:0 ]  p;
    bit  [ 255:0 ]  q;
    bit  [ 511:0 ]  e; 
} test_data_t;


//function int inverse_modulo(input int e_value, n_value);
//    automatic int d_temp = 1;
//    while (((e_value * d_temp) % n_value) != 1) begin
//        d_temp++;
//    end
//    return d_temp;
//endfunction


`SVUNIT_TESTS_BEGIN

    `SVTEST(simple_write)
        int iteration;
        int iteration_cntr = 0;

        test_data_t test_data;
        bit in_progress = 0;

        bit  [511:0]  expected_result_n;
        bit  [511:0]  expected_result_d;

        iteration =  1; //$urandom_range(10,20); // change to 1 000 000 - 10 000 000

        $display("iterations: %0d", iteration);

        while(1) begin
            @(posedge clk_100mhz);
            if(in_progress == 0) begin

                //for(int i = 0; i < 16; i++) begin
                //    if(i < 8) begin
                //        test_data.p[32*i +: 32] = $urandom();
                //        test_data.q[32*i +: 32] = $urandom();
                //    end
                //    
                //    test_data.e[32*i +: 32] = $urandom();
                //end
                
                test_data.p = 256'd3;
                test_data.q = 256'd11;
                test_data.e = 512'd3;

                p <= test_data.p;
                q <= test_data.q;
                e <= test_data.e;
                enable <= 1'b1;
                @(posedge clk_100mhz);
                enable <= 1'b0;
                in_progress = 1;
            end

            if(in_progress == 1 && result_valid == 1) begin

                expected_result_n = p * q;
                //expected_result_d = inverse_modulo(e, expected_result_n);

                if(expected_result_n !== n ) begin
                    $display("[%0d] expected result n = %0x !== n = %0x", iteration_cntr, expected_result_n, n);
                end

                `FAIL_UNLESS_EQUAL(expected_result_n, n)

                //if(expected_result_d !== d ) begin
                //    $display("[%0d] expected result d = %0x !== d = %0x", iteration_cntr, expected_result_d, d);
                //end

                if(7 !== d ) begin
                    $display("[%0d] expected result d = %0x !== d = %0x", iteration_cntr, expected_result_d, d);
                end

                `FAIL_UNLESS_EQUAL(7, d) // change to expected result

                $display("status: %0d / %0d", iteration_cntr, iteration);

                if(iteration_cntr == iteration) begin
                    iteration_cntr = 0;
                    break;
                end
                else begin
                    iteration_cntr++;
                end
                
                in_progress = 0;
            end
        end

    `SVTEST_END
`SVUNIT_TESTS_END

endmodule