`timescale 1 ns / 1 ns

`include "svunit_defines.svh"

module mnozenie_unit_test;
    import svunit_pkg::svunit_testcase;
    import unit_test_pkg::*;
    import registers_pkg::*;
    import axi4_lite_pkg::*;

    string name = "mnozenie_unit_test";
    svunit_testcase svunit_ut;

    localparam real TIME_BASE = 1000.0;
    localparam real CLOCK_100 = (TIME_BASE / 100.00);

    logic reset_n    = 1'b0;
    logic clk_100mhz = 0;

    initial forever #(CLOCK_100/2) clk_100mhz = ~clk_100mhz;

    // ----------------------------------
    // ------- DUT initialization -------
    // ----------------------------------
    
    logic [ 511:0] a = '0;
    logic [ 511:0] b = '0;
    logic          enable = '0;
    logic          result_valid;
    logic [1023:0] result;

    mnozenie dut(
        .rst            (~reset_n),
        .clk            (clk_100mhz),
        .a              (a),
        .b              (b),
        .enable         (enable),
        .result_valid   (result_valid),
        .result         (result)
    );

    function void build();
        svunit_ut = new(name);
    endfunction

    task setup();
        svunit_ut.setup();
        reset_n = 1'b0;

        @(posedge clk_100mhz);
        reset_n = 1'b1;
    endtask

    task teardown();
        svunit_ut.teardown();
        reset_n = 1'b0;
    endtask


typedef struct packed{
    bit  [ 511:0  ]  a;
    bit  [ 511:0  ]  b; 
} test_data_t;


`SVUNIT_TESTS_BEGIN

    `SVTEST(simple_write)
        int iteration;
        int iteration_cntr = 0;

        test_data_t test_data;
        bit in_progress = 0;

        bit  [ 1023:0 ]  expected_result;

        iteration = $urandom_range(100000,1000000);

        $display("iterations: %0d", iteration);

        while(1) begin
            @(posedge clk_100mhz);
            if(in_progress == 0) begin
                for(int i = 0; i < 16; i++) begin
                    test_data.a[32*i +: 32] = $urandom();
                    test_data.b[32*i +: 32] = $urandom();
                end
                
                a <= test_data.a;
                b <= test_data.b;
                enable <= 1'b1;
                @(posedge clk_100mhz);
                enable <= 1'b0;
                in_progress = 1;
            end

            if(in_progress == 1 && result_valid == 1) begin
                expected_result = test_data.a * test_data.b;

                if(expected_result !== result) begin
                    $display("[%0d] expected result = %0x !== result = %0x", iteration_cntr, expected_result, result);
                end

                `FAIL_UNLESS_EQUAL(expected_result, result)

                $display("status: %0d / %0d", iteration_cntr, iteration);

                if(iteration_cntr == iteration) begin
                    iteration_cntr = 0;
                    break;
                end
                else begin
                    iteration_cntr++;
                end
                
                in_progress = 0;
            end
        end

    `SVTEST_END
`SVUNIT_TESTS_END

endmodule