`define N(n)                       [(n)-1:0]
`define FFx(signal,bits)           always @ ( posedge clk or posedge  rst ) if (   rst   )  signal <= bits;  else
`define XLEN   1024
`define DEL     2

module generator(
    input   logic               clk,
    input   logic               rst,
    input   logic  [255:0]      p,
    input   logic  [255:0]      q,
    input   logic  [511:0]      e,
    input   logic               enable,
    output  logic               result_valid,
    output  logic  [511:0]      n,
    output  logic  [511:0]      d
);


//====================================================
//=======  FSM Parameters  ===========================
//====================================================
typedef enum logic [3:0] {
    IDLE              = 4'b0000,
    PQ_MULT_STATE     = 4'b0001,
    PQ_SUB_STATE      = 4'b0010,
    PHI_MULT_STATE    = 4'b0011,
    D_INC_STATE       = 4'b0100,
    ED_MULT_STATE     = 4'b0101,
    DIV_STATE         = 4'b0110,
    CHECK_STATE       = 4'b0111,
    RESULT_STATE      = 4'b1000
} main_fsm_t;

main_fsm_t main_fsm;

logic   [511:0 ]    d_temp_pre_inc    ;
logic   [511:0 ]    d_temp_post_inc   ;
logic   [511:0 ]    n_value           ;
logic   [255:0 ]    p_sub             ;
logic   [255:0 ]    q_sub             ;
logic   [511:0 ]    phi               ;

logic               mult_enable_pq    ;
logic   [511:0 ]    mult_result_pq    ;
logic               mult_valid_pq     ;

logic               mult_enable_ed    ;
logic   [1023:0]    mult_result_ed    ;
logic               mult_valid_ed     ;

logic               mult_enable_phi   ;
logic   [511:0 ]    mult_result_phi   ;
logic               mult_valid_phi    ;

logic               div_enable        ;
logic               div_busy          ;
logic               div_done          ;
logic               div_dbz           ;
logic   [1023:0 ]   div_result        ; // bylo 512
logic   [1023:0 ]   div_remainder     ; // bylo 512
logic               div_valid         ;


logic               increment_enable  ;
logic               increment_clear   ;
logic               increment_valid   ;

logic               sub_clear         ;
logic               sub_enable        ;

logic               sub_valid_p       ;
logic    [255:0 ]   sub_result_p      ;

logic               sub_valid_q       ;
logic    [255:0 ]   sub_result_q      ;

assign n_value = mult_result_pq;
assign p_sub = sub_result_p;
assign q_sub = sub_result_q;
assign phi = mult_result_phi;


mult u_mult_pq(
    .clock           (  clk              ),
    .reset           (  rst              ),
    .enable          (  mult_enable_pq   ),
    .dataa           (  p                ),
    .datab           (  q                ),
    .result          (  mult_result_pq   ),
    .result_valid    (  mult_valid_pq    )
);

mult u_mult_phi(
    .clock           (  clk              ),
    .reset           (  rst              ),
    .enable          (  mult_enable_phi  ),
    .dataa           (  p_sub            ),
    .datab           (  q_sub            ),
    .result          (  mult_result_phi  ),
    .result_valid    (  mult_valid_phi   )
);

odejmowanie u_sub_p(
    .rst            (rst),
    .clk            (clk),
    .a              (p),
    .b              (256'b1),
    .enable         (sub_enable),
    .clear          (sub_clear),
    .result_valid   (sub_valid_p),
    .result         (sub_result_p)
);

odejmowanie u_sub_q(
    .rst            (rst),
    .clk            (clk),
    .a              (q),
    .b              (256'b1),
    .enable         (sub_enable),
    .clear          (sub_clear),
    .result_valid   (sub_valid_q),
    .result         (sub_result_q)
);

mnozenie u_mnozenie_ed(
    .rst            (  rst               ),
    .clk            (  clk               ),
    .a              (  e                 ),
    .b              (  d_temp_post_inc   ),
    .enable         (  mult_enable_ed    ),
    .result_valid   (  mult_valid_ed     ),
    .result         (  mult_result_ed    )
);

localparam WIDTH = 1024;

divider #(
        .WIDTH  (WIDTH)
) u_div ( 
        .clk          (clk),
        .rst          (rst),
        .start        (div_enable),
        .busy         (div_busy),
        .done         (div_done),
        .valid        (div_valid),
        .dbz          (div_dbz),
        .a            (mult_result_ed),
        .b            ({512'b0, phi}),
        .val          (div_result),
        .rem          (div_remainder)
    );


increment u_increment(
    .rst            (  rst                ),
    .clk            (  clk                ),
    .value          (  d_temp_pre_inc     ),
    .enable         (  increment_enable   ),
    .clear          (  increment_clear    ),
    .result_valid   (  increment_valid    ),
    .result         (  d_temp_post_inc    )
);

//====================================================
//=======  Main FSM  ==================
//====================================================
always_ff @(posedge clk or posedge rst) begin

    if (rst) begin
        main_fsm <= IDLE;

        d_temp_pre_inc   <= '0;
        mult_enable_pq   <= '0;
        mult_enable_ed   <= '0;
        mult_enable_phi  <= '0;
        div_enable       <= '0;
        increment_enable <= '0;
        increment_clear  <= '0;
        sub_clear        <= '0;
        sub_enable       <= '0;

    end 

    else begin
        case (main_fsm) 

            IDLE: begin

                if (enable == 1'b1) begin
                    main_fsm      <= PQ_MULT_STATE;
                    mult_enable_pq   <= 1'b1;
                end

                d_temp_pre_inc   <= '0;
                mult_enable_ed   <= '0;
                mult_enable_phi  <= '0;
                div_enable       <= '0;
                increment_enable <= '0;
                increment_clear  <= '1;
                sub_clear        <= '1;
                sub_enable       <= '0;

            end

            PQ_MULT_STATE: begin

                if (mult_valid_pq == 1'b1) begin
                    main_fsm   <= PQ_SUB_STATE;
                    sub_enable <= '1;
                end

                mult_enable_pq   <= 1'b0;
                sub_clear  <= '0;
                
            end

            PQ_SUB_STATE: begin

                if (sub_valid_p == 1'b1 && sub_valid_q == 1'b1) begin
                    main_fsm        <= PHI_MULT_STATE;
                    mult_enable_phi <= 1'b1;
                end

                sub_enable   <= 1'b0;
                
            end

            PHI_MULT_STATE: begin

                if (mult_valid_phi == 1'b1) begin
                    main_fsm         <= D_INC_STATE;
                    increment_enable <= 1'b1;
                end

                mult_enable_phi  <= 1'b0;
                increment_clear  <= '0;
                
            end

            D_INC_STATE : begin

                if(increment_valid == 1'b1) begin
                    main_fsm       <= ED_MULT_STATE;
                    mult_enable_ed <= 1'b1;
                    d_temp_pre_inc <= d_temp_post_inc;
                end

                increment_enable <= 1'b0;

            end

            ED_MULT_STATE: begin

                if (mult_valid_ed == 1'b1) begin
                    main_fsm   <= DIV_STATE;
                    div_enable <= 1'b1;
                end

                mult_enable_ed <= 1'b0;
                
            end

            DIV_STATE : begin

                if(div_done == 1'b1) begin
                    main_fsm <= CHECK_STATE;
                end

                div_enable <= 1'b0;

            end

            CHECK_STATE : begin

                if(div_remainder == 'd1) begin
                    main_fsm <= RESULT_STATE;
                    d            <= d_temp_post_inc;
                    n            <= n_value;
                    result_valid <= 1'b1;
                end
                else begin
                    main_fsm         <= D_INC_STATE;
                    increment_enable <= 1'b1;
                end

            end

            RESULT_STATE : begin

                if (enable == 1'b0) begin
                    main_fsm <= IDLE;
                end

            end

            default: begin

                main_fsm <= IDLE;

                d_temp_pre_inc   <= '0;
                mult_enable_pq   <= '0;
                mult_enable_ed   <= '0;
                mult_enable_phi  <= '0;
                div_enable       <= '0;
                increment_enable <= '0;
                increment_clear  <= '0;
                sub_clear        <= '0;
                sub_enable       <= '0;
                
            end
        endcase
    end
end


endmodule