module modul_studenta (
    input  logic        rst,
    input  logic        clk,

    output logic        s_axil_awready,
    input  wire         s_axil_awvalid,
    input  wire [20:0]  s_axil_awaddr,
    input  wire [2:0]   s_axil_awprot,
    output logic        s_axil_wready,
    input  wire         s_axil_wvalid,
    input  wire [31:0]  s_axil_wdata,
    input  wire [3:0]   s_axil_wstrb,
    input  wire         s_axil_bready,
    output logic        s_axil_bvalid,
    output logic [1:0]  s_axil_bresp,
    output logic        s_axil_arready,
    input  wire         s_axil_arvalid,
    input  wire [20:0]  s_axil_araddr,
    input  wire [2:0]   s_axil_arprot,
    input  wire         s_axil_rready,
    output logic        s_axil_rvalid,
    output logic [31:0] s_axil_rdata,
    output logic [1:0]  s_axil_rresp,

    output logic [7:0]  LED //delete
);



import registers_pkg::*;

registers_pkg::registers__out_t hwif_out;
registers_pkg::registers__in_t hwif_in;

//====================================================
//=======  FSM Parameters  ===========================
//====================================================

parameter       KEY_GEN_SIZE = 2  ;

parameter       KEY_GEN_IDLE = 2'b00,
                KEY_GEN_ENABLE_STATE = 2'b01,
                KEY_GEN_STATE = 2'b10,
                KEY_GEN_RESULT_STATE = 2'b11;



logic   [KEY_GEN_SIZE-1:0]      key_gen_state;
logic   [KEY_GEN_SIZE-1:0]      key_gen_next_state;

logic   [7:0]               LEDLogic     ; //delete
logic                       FSM_enable   ;

logic   [255:0]              p_value;
logic   [255:0]              q_value;
logic   [511:0]              e_value;

logic   [511:0]              d_value;
logic   [511:0]              n_value;

logic                       key_gen_enable;
logic                       key_gen_valid;
logic   [511:0 ]            key_gen_result;

//assign LED[0] = hwif_out.LEDS.LED0.value;
//assign LED[7] = 1'b1;

assign LED = LEDLogic; //deletes
assign FSM_enable = hwif_out.FSM.ENABLE.value;

assign p_value = {hwif_out.P_PART8.P_PART.value, hwif_out.P_PART7.P_PART.value, hwif_out.P_PART6.P_PART.value,
                hwif_out.P_PART5.P_PART.value, hwif_out.P_PART4.P_PART.value, hwif_out.P_PART3.P_PART.value,
                hwif_out.P_PART2.P_PART.value, hwif_out.P_PART1.P_PART.value};

assign q_value = {hwif_out.Q_PART8.Q_PART.value, hwif_out.Q_PART7.Q_PART.value, hwif_out.Q_PART6.Q_PART.value,
                hwif_out.Q_PART5.Q_PART.value, hwif_out.Q_PART4.Q_PART.value, hwif_out.Q_PART3.Q_PART.value,
                hwif_out.Q_PART2.Q_PART.value, hwif_out.Q_PART1.Q_PART.value};

assign e_value = {hwif_out.E_PART16.E_PART.value, hwif_out.E_PART15.E_PART.value, hwif_out.E_PART14.E_PART.value, 
                hwif_out.E_PART13.E_PART.value, hwif_out.E_PART12.E_PART.value,
                hwif_out.E_PART11.E_PART.value, hwif_out.E_PART10.E_PART.value, hwif_out.E_PART9.E_PART.value,
                hwif_out.E_PART8.E_PART.value, hwif_out.E_PART7.E_PART.value, hwif_out.E_PART6.E_PART.value,
                hwif_out.E_PART5.E_PART.value, hwif_out.E_PART4.E_PART.value, hwif_out.E_PART3.E_PART.value,
                hwif_out.E_PART2.E_PART.value, hwif_out.E_PART1.E_PART.value};



//------------------------------------------
//------------- Registers ------------------
//------------------------------------------
registers u_registers (
    .clk                (clk),
    .rst                (rst),

    .s_axil_awready     (s_axil_awready),
    .s_axil_awvalid     (s_axil_awvalid),
    .s_axil_awaddr      (s_axil_awaddr[8:0]),
    .s_axil_awprot      (s_axil_awprot),

    .s_axil_wready      (s_axil_wready),
    .s_axil_wvalid      (s_axil_wvalid),
    .s_axil_wdata       (s_axil_wdata),
    .s_axil_wstrb       (s_axil_wstrb),

    .s_axil_bready      (s_axil_bready),
    .s_axil_bvalid      (s_axil_bvalid),
    .s_axil_bresp       (s_axil_bresp),

    .s_axil_arready     (s_axil_arready),
    .s_axil_arvalid     (s_axil_arvalid),
    .s_axil_araddr      (s_axil_araddr[8:0]),
    .s_axil_arprot      (s_axil_arprot),

    .s_axil_rready      (s_axil_rready),
    .s_axil_rvalid      (s_axil_rvalid),
    .s_axil_rdata       (s_axil_rdata),
    .s_axil_rresp       (s_axil_rresp),

    .hwif_in            (hwif_in),
    .hwif_out           (hwif_out)
);


generator u_generator(
    .clk                (clk),
    .rst                (rst),
    .p                  (p_value),
    .q                  (q_value),
    .e                  (e_value),
    .enable             (key_gen_enable),
    .result_valid       (key_gen_valid),
    .n                  (n_value),
    .d                  (d_value)
);

//====================================================
//=======  FSM KEY GENERATOR STATE REGISTER  =======================
//====================================================
always @ (posedge clk)
 begin : FSM_SEQ_KEY_GEN
   if (rst == 1'b1) begin
        key_gen_state <=  KEY_GEN_IDLE;
   end else begin
        key_gen_state <=  key_gen_next_state;
   end
 end



 //====================================================
//=======  FSM KEY GENERATOR STATE LOGIC  ==========================
//====================================================
always_comb
 begin : FSM_COMBO_KEY_GEN
  
  key_gen_next_state = KEY_GEN_IDLE;
  
  case(key_gen_state)
    KEY_GEN_IDLE :  if (FSM_enable == 1'b1) begin
                key_gen_next_state = KEY_GEN_ENABLE_STATE;
            end else begin
                key_gen_next_state = KEY_GEN_IDLE;
            end

    KEY_GEN_ENABLE_STATE :  if (key_gen_enable == 1'b1) begin
                key_gen_next_state = KEY_GEN_STATE;
            end else begin
                key_gen_next_state = KEY_GEN_ENABLE_STATE;
            end

    KEY_GEN_STATE :  if (key_gen_valid == 1'b1) begin
                key_gen_next_state = KEY_GEN_RESULT_STATE;
            end else begin
                key_gen_next_state = KEY_GEN_STATE;
            end

    KEY_GEN_RESULT_STATE :  if (FSM_enable == 1'b0) begin
                key_gen_next_state = KEY_GEN_IDLE;
            end else begin
                key_gen_next_state = KEY_GEN_RESULT_STATE;
            end

    default : key_gen_next_state = KEY_GEN_IDLE;
   endcase
 end 



//====================================================
//=======  FSM KEY GENERATOR OUTPUT LOGIC  ==========================
//====================================================
always_comb
begin : OUTPUT_LOGIC_KEY_GEN
  case(key_gen_state)
   KEY_GEN_IDLE : begin
        key_gen_enable = 1'b0;

        {hwif_in.N_PART16.N_PART.next,
        hwif_in.N_PART15.N_PART.next,
        hwif_in.N_PART14.N_PART.next,
        hwif_in.N_PART13.N_PART.next,
        hwif_in.N_PART12.N_PART.next,
        hwif_in.N_PART11.N_PART.next,
        hwif_in.N_PART10.N_PART.next,
        hwif_in.N_PART9.N_PART.next,
        hwif_in.N_PART8.N_PART.next,
        hwif_in.N_PART7.N_PART.next,
        hwif_in.N_PART6.N_PART.next,
        hwif_in.N_PART5.N_PART.next,
        hwif_in.N_PART4.N_PART.next,
        hwif_in.N_PART3.N_PART.next,
        hwif_in.N_PART2.N_PART.next,
        hwif_in.N_PART1.N_PART.next} = 512'b0;

        {hwif_in.D_PART16.D_PART.next,
        hwif_in.D_PART15.D_PART.next,
        hwif_in.D_PART14.D_PART.next,
        hwif_in.D_PART13.D_PART.next,
        hwif_in.D_PART12.D_PART.next,
        hwif_in.D_PART11.D_PART.next,
        hwif_in.D_PART10.D_PART.next,
        hwif_in.D_PART9.D_PART.next,
        hwif_in.D_PART8.D_PART.next,
        hwif_in.D_PART7.D_PART.next,
        hwif_in.D_PART6.D_PART.next,
        hwif_in.D_PART5.D_PART.next,
        hwif_in.D_PART4.D_PART.next,
        hwif_in.D_PART3.D_PART.next,
        hwif_in.D_PART2.D_PART.next,
        hwif_in.D_PART1.D_PART.next} = 512'b0;
    end
   
   KEY_GEN_ENABLE_STATE : begin
        key_gen_enable = 1'b1;
    end

   KEY_GEN_STATE : begin
        {hwif_in.N_PART16.N_PART.next,
        hwif_in.N_PART15.N_PART.next,
        hwif_in.N_PART14.N_PART.next,
        hwif_in.N_PART13.N_PART.next,
        hwif_in.N_PART12.N_PART.next,
        hwif_in.N_PART11.N_PART.next,
        hwif_in.N_PART10.N_PART.next,
        hwif_in.N_PART9.N_PART.next,
        hwif_in.N_PART8.N_PART.next,
        hwif_in.N_PART7.N_PART.next,
        hwif_in.N_PART6.N_PART.next,
        hwif_in.N_PART5.N_PART.next,
        hwif_in.N_PART4.N_PART.next,
        hwif_in.N_PART3.N_PART.next,
        hwif_in.N_PART2.N_PART.next,
        hwif_in.N_PART1.N_PART.next} = n_value;

        {hwif_in.D_PART16.D_PART.next,
        hwif_in.D_PART15.D_PART.next,
        hwif_in.D_PART14.D_PART.next,
        hwif_in.D_PART13.D_PART.next,
        hwif_in.D_PART12.D_PART.next,
        hwif_in.D_PART11.D_PART.next,
        hwif_in.D_PART10.D_PART.next,
        hwif_in.D_PART9.D_PART.next,
        hwif_in.D_PART8.D_PART.next,
        hwif_in.D_PART7.D_PART.next,
        hwif_in.D_PART6.D_PART.next,
        hwif_in.D_PART5.D_PART.next,
        hwif_in.D_PART4.D_PART.next,
        hwif_in.D_PART3.D_PART.next,
        hwif_in.D_PART2.D_PART.next,
        hwif_in.D_PART1.D_PART.next} = d_value;

        key_gen_enable = 1'b0;
    end

   KEY_GEN_RESULT_STATE : begin
        //odczyt z rejestru 
        key_gen_enable = 1'b0;
    end

   default : begin
        {hwif_in.N_PART16.N_PART.next,
        hwif_in.N_PART15.N_PART.next,
        hwif_in.N_PART14.N_PART.next,
        hwif_in.N_PART13.N_PART.next,
        hwif_in.N_PART12.N_PART.next,
        hwif_in.N_PART11.N_PART.next,
        hwif_in.N_PART10.N_PART.next,
        hwif_in.N_PART9.N_PART.next,
        hwif_in.N_PART8.N_PART.next,
        hwif_in.N_PART7.N_PART.next,
        hwif_in.N_PART6.N_PART.next,
        hwif_in.N_PART5.N_PART.next,
        hwif_in.N_PART4.N_PART.next,
        hwif_in.N_PART3.N_PART.next,
        hwif_in.N_PART2.N_PART.next,
        hwif_in.N_PART1.N_PART.next} = 512'b0;

        {hwif_in.D_PART16.D_PART.next,
        hwif_in.D_PART15.D_PART.next,
        hwif_in.D_PART14.D_PART.next,
        hwif_in.D_PART13.D_PART.next,
        hwif_in.D_PART12.D_PART.next,
        hwif_in.D_PART11.D_PART.next,
        hwif_in.D_PART10.D_PART.next,
        hwif_in.D_PART9.D_PART.next,
        hwif_in.D_PART8.D_PART.next,
        hwif_in.D_PART7.D_PART.next,
        hwif_in.D_PART6.D_PART.next,
        hwif_in.D_PART5.D_PART.next,
        hwif_in.D_PART4.D_PART.next,
        hwif_in.D_PART3.D_PART.next,
        hwif_in.D_PART2.D_PART.next,
        hwif_in.D_PART1.D_PART.next} = 512'b0;
        
        key_gen_enable = 1'b0;
    end
  endcase
end 

//====================================================
//=======  KEY GENERATION  ==========================
//====================================================





`ifdef FSM_DEBUG

(* noprune *) logic [KEY_GEN_SIZE-1:0]  DEBUG_state        ;// Seq part of the FSM
(* noprune *) logic [KEY_GEN_SIZE-1:0]  DEBUG_next_state   ;// combo part of FSM
(* noprune *) logic [7:0]       DEBUG_LEDLogic     ;
(* noprune *) logic             DEBUG_FSM_enable   ;


always_ff @(posedge clk) begin

    DEBUG_state       <= key_gen_state      ;
    DEBUG_next_state  <= key_gen_next_state  ;
    DEBUG_LEDLogic    <= LEDLogic    ;
    DEBUG_FSM_enable  <= FSM_enable  ;
end
`endif






endmodule