
module szyfrowanie(
    input   logic               clk,
    input   logic               rst,
    input   logic  [511:0]      block,
    input   logic  [511:0]      n,
    input   logic  [511:0]      e,
    input   logic               enable,
    output  logic               result_valid,
    output  logic  [511:0]      c
);


//====================================================
//=======  FSM Parameters  ===========================
//====================================================
typedef enum logic [2:0] {
    IDLE              = 3'b000,
    EPRIM_INC_STATE   = 3'b001,
    XC_MULT_STATE     = 3'b010,
    DIV_STATE         = 3'b011,
    CHECK_STATE       = 3'b100,
    RESULT_STATE      = 3'b101
} main_fsm_t;

main_fsm_t main_fsm;

logic   [511:0 ]    eprim_temp_pre_inc    ;
logic   [511:0 ]    eprim_temp_post_inc   ;
logic   [511:0 ]    c_result              ;
logic   [1023:0]    c_result_pom          ;

logic               mult_enable_xc    ;
logic   [1023:0]    mult_result_xc    ;
logic               mult_valid_xc     ;

logic               div_enable        ;
logic               div_busy          ;
logic               div_done          ;
logic               div_dbz           ;
logic   [1023:0 ]   div_result        ; // bylo 512
logic   [1023:0 ]   div_remainder     ; // bylo 512
logic               div_valid         ;

logic               increment_enable  ;
logic               increment_clear   ;
logic               increment_valid   ;


mnozenie u_mnozenie_xc(
    .rst            (  rst                  ),
    .clk            (  clk                  ),
    .a              (  block                ),
    .b              (  c_result             ),
    .enable         (  mult_enable_xc       ),
    .result_valid   (  mult_valid_xc        ),
    .result         (  mult_result_xc       )
);

localparam WIDTH = 1024;

divider #(
        .WIDTH  (WIDTH)
) u_div ( 
        .clk          ( clk             ),
        .rst          ( rst             ),
        .start        ( div_enable      ),
        .busy         ( div_busy        ),
        .done         ( div_done        ),
        .valid        ( div_valid       ),
        .dbz          ( div_dbz         ),
        .a            ( mult_result_xc  ),
        .b            ( n               ),
        .val          ( div_result      ),
        .rem          ( c_result_pom    )
    );

increment u_increment(
    .rst            (  rst                ),
    .clk            (  clk                ),
    .value          (  eprim_temp_pre_inc ),
    .enable         (  increment_enable   ),
    .clear          (  increment_clear    ),
    .result_valid   (  increment_valid    ),
    .result         (  eprim_temp_post_inc)
);

//====================================================
//=======  Main FSM  ==================
//====================================================
always_ff @(posedge clk or posedge rst) begin

    if (rst) begin
        main_fsm <= IDLE;

        c_result            <= 512'b1;
        eprim_temp_pre_inc  <= '0;
        mult_enable_xc      <= '0;
        div_enable          <= '0;
        increment_enable    <= '0;
        increment_clear     <= '0;

    end 

    else begin
        case (main_fsm) 

            IDLE: begin

                if (enable == 1'b1) begin
                    main_fsm        <= CHECK_STATE;
                end

                eprim_temp_pre_inc  <= '0;
                mult_enable_xc      <= '0;
                div_enable          <= '0;
                increment_enable    <= '0;
                increment_clear     <= '0;

            end

            CHECK_STATE : begin

                if(eprim_temp_post_inc < e) begin
                    main_fsm         <= EPRIM_INC_STATE;
                    increment_enable <= 1'b1;
                end
                else begin
                    main_fsm     <= RESULT_STATE;
                    c            <= c_result;
                    result_valid <= 1'b1;
                end

            end

            EPRIM_INC_STATE : begin

                if(increment_valid == 1'b1) begin
                    main_fsm           <= XC_MULT_STATE;
                    mult_enable_xc     <= 1'b1;
                    eprim_temp_pre_inc <= eprim_temp_post_inc;
                end

                increment_enable <= 1'b0;

            end

            XC_MULT_STATE: begin

                if (mult_valid_xc == 1'b1) begin
                    main_fsm   <= DIV_STATE;
                    div_enable <= 1'b1;
                end

                mult_enable_xc <= 1'b0;
                
            end

            DIV_STATE : begin

                if(div_done == 1'b1) begin
                    main_fsm <= CHECK_STATE;
                    c_result <= c_result_pom[511:0];//przypisac mlodsza czesc
                end

                div_enable <= 1'b0;

            end

            RESULT_STATE : begin

                if (enable == 1'b0) begin
                    main_fsm <= IDLE;
                end

            end

            default: begin

                main_fsm <= IDLE;

                eprim_temp_pre_inc  <= '0;
                mult_enable_xc      <= '0;
                div_enable          <= '0;
                increment_enable    <= '0;
                increment_clear     <= '0;
                
            end
        endcase
    end
end


endmodule